==> oscars_mgmnt-0.1.79.dev20240913+132-py3-none-any.whl: oscars_mgmnt/parser/es/es.py <==
import re
import time

import pyproj

from oscars_mgmnt.file_object import ObjectInfo, FileInfo
from oscars_mgmnt.geometry.polygon import EsaPolygon
from oscars_mgmnt.parser.base import BaseParser
from oscars_mgmnt.utils.es_rest_support import ElasticRestSupport


class ElasticsearchParser(BaseParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        BaseParser.__init__(self, template)
        self.es_rest_support = ElasticRestSupport(hosts=es_source_host)
        self.es_index = es_source_index

        # regex used to retrieve the product identifier:
        #   retrieve product identifier based on folder name
        #   look for a string that contains date information, like 20200727T173023
        self.regex_identifier = re.compile(r".*/(\w*_\d{8}T\d{6}_\w*)(?:($)|(/.*))")

    def init_dict(self):
        self.dict_info = {
            "title": "title",
            "geometry": "geometry",
            "footprint": "properties.footprint",
            "orbitnumber": "properties.orbitnumber",
            "orbitdirection": "properties.orbitdirection",
            "relativeorbitnumber": "properties.relativeorbitnumber",
            "beginposition": "properties.beginposition",
            "endposition": "properties.endposition",
            "nominaldatetime": "properties.beginposition",
            "updatetime": "properties.processing_status.processing_end",
            "producttype": "properties.producttype",
            "sensormode": "properties.sensoroperationalmode",
            "processingmode": "properties.processingmode",
            "timeliness": "properties.timeliness",
            "inputs": "inputs"
        }

    def convert(self, product_file_info):
        """ retrieves the product metadata from Elasticsearch based on the product identifier
        :param product_file_info: the description of the files associated to a product
        :type product_file_info: ObjectInfo
        """

        # retrieve product identifier using regex pattern
        match = self.regex_identifier.search(product_file_info.get_product_path())

        if match and len(match.groups()) > 0:
            product_identifier = match.group(1)
        else:
            raise Exception("Could not derive the product identifier from the product path {}"
                              .format(product_file_info.get_product_path()))

        # get document from Elasticsearch
        metadata = self.get_metadata(product_identifier)

        # Initialize values dict
        values = {}
        values['id'] = product_identifier

        # Loop over all the property names and add them to the values if they are present
        for k, v in self.dict_info.items():
            tmp = self.extract_value(metadata, v)
            if tmp is not None:
                values[k] = tmp

        # Generate other properties based on extracted values
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())

        specific_infos = self.splitted_treatment(product_file_info, values)
        self.fix_id(values)

        # Generation of the output
        results = {}

        if self.jinja_template is None:
            self.with_template(self.template)

        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def get_metadata(self, product_identifier):
        """ Get the product metadata from Elasticsearch
        :param product_identifier: product identifier
        :type product_identifier: str
        """
        return self.es_rest_support.get(self.es_index, product_identifier)

    def extract_value(self, metadata, path):
        """
        Extract a property, given by the path, from the metadata
        :param metadata: the metadata
        :type metadata: dict
        :param path: path of the of the property in the metadata dict
        :type path: str
        """
        path = path.split(".")
        tmp = metadata
        for key in path:
            if key in tmp:
                tmp = tmp[key]
            else:
                return None
        return tmp

    def fix_id(self, values):
        """ Fix the ID of the product by prepending the parent identifier
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """
        values['id'] = values['parentidentifier'] + ":" + values['title']

    def fix_geometry(self, values, simplify=True):
        """ Simplify the geometry and generate the bbox
        :param values: dictionary containing the values, must include the geometry
        :type values: dict
        """
        if "geometry" in values:
            polygon = EsaPolygon().from_geojson(values['geometry'], simplify=simplify)
        else:
            polygon = EsaPolygon().from_wkt(values['footprint'], simplify=simplify)
        values['geometry'] = polygon.as_geojson_string()
        values['westbbox'], values['southbbox'], values['eastbbox'], values['northbbox'] = polygon.get_bbox()

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        uid = values['title']
        if "_V" in uid:
            values['productVersion'] = uid[uid.rfind("_V")+1:]

        self.fix_products(product_file_info)

        return uid

    def fix_products(self, product_file_info):
        """" Helper method that moves files in the product file info to the correct section
        :param product_file_info: product file info
        :type product_file_info: ObjectInfo
        """
        raise NotImplementedError("Implement this method in a subclass.")

    def add_wms_quicklook(self, wms_layer: str, product_file_info: ObjectInfo, values: dict):
        west, south = TRANSFORM_4326_TO_3857.transform(values['westbbox'], values['southbbox'])
        east, north = TRANSFORM_4326_TO_3857.transform(values['eastbbox'], values['northbbox'])

        # in WMS 1.3.0, the coordinate order is lat-lon
        wms_link = f"https://services.terrascope.be/wms/v2" \
                   "?SERVICE=WMS" \
                   "&REQUEST=getMap" \
                   "&VERSION=1.3.0" \
                   "&CRS=EPSG:3857" \
                   "&SRS=EPSG:3857" \
                   f"&LAYERS={wms_layer}" \
                   f"&TIME={values['endposition'][:10]}" \
                   f"&BBOX={west},{south},{east},{north}" \
                   "&WIDTH=80" \
                   "&HEIGHT=80" \
                   "&FORMAT=image/png" \
                   "&TRANSPARENT=true"

        wms_quicklook = FileInfo(wms_link, file_type="image/png", file_title="WMS")
        product_file_info.previews.append(wms_quicklook)

    @staticmethod
    def format_uid_date(uid_date):
        return '{}-{}-{}T{}:{}:{}Z'.format(uid_date[0:4], uid_date[4:6], uid_date[6:8],
                                           uid_date[9:11], uid_date[11:13], uid_date[13:15])


TRANSFORM_4326_TO_3857 = pyproj.Transformer.from_crs("EPSG:4326", "EPSG:3857", always_xy=True)

==> oscars_mgmnt-0.1.82.dev20241203+145-py3-none-any.whl: oscars_mgmnt/parser/es/es.py <==
import re
import time

import pyproj

from oscars_mgmnt.file_object import ObjectInfo, FileInfo
from oscars_mgmnt.geometry.polygon import EsaPolygon
from oscars_mgmnt.parser.base import BaseParser
from oscars_mgmnt.utils.es_rest_support import ElasticRestSupport


class ElasticsearchParser(BaseParser):

    def __init__(self, template=None, es_source_host=None, es_source_index=None):
        BaseParser.__init__(self, template)
        self.es_rest_support = ElasticRestSupport(hosts=es_source_host)
        self.es_index = es_source_index

        # regex used to retrieve the product identifier:
        #   retrieve product identifier based on folder name
        #   look for a string that contains date information, like 20200727T173023
        self.regex_identifier = re.compile(r".*/(\w*_\d{8}T\d{6}_\w*)(?:($)|(/.*))")

    def init_dict(self):
        self.dict_info = {
            "title": "title",
            "geometry": "geometry",
            "footprint": "properties.footprint",
            "orbitnumber": "properties.orbitnumber",
            "orbitdirection": "properties.orbitdirection",
            "relativeorbitnumber": "properties.relativeorbitnumber",
            "beginposition": "properties.beginposition",
            "endposition": "properties.endposition",
            "nominaldatetime": "properties.beginposition",
            "updatetime": "properties.processing_status.processing_end",
            "producttype": "properties.producttype",
            "sensormode": "properties.sensoroperationalmode",
            "processingmode": "properties.processingmode",
            "timeliness": "properties.timeliness",
            "inputs": "inputs"
        }

    def get_product_identifier(self, product_file_info):
        # retrieve product identifier using regex pattern
        match = self.regex_identifier.search(product_file_info.get_product_path())

        if match and len(match.groups()) > 0:
            product_identifier = match.group(1)
        else:
            raise Exception("Could not derive the product identifier from the product path {}"
                            .format(product_file_info.get_product_path()))
        return product_identifier


    def convert(self, product_file_info):
        """ retrieves the product metadata from Elasticsearch based on the product identifier
        :param product_file_info: the description of the files associated to a product
        :type product_file_info: ObjectInfo
        """

        product_identifier = self.get_product_identifier(product_file_info)

        # get document from Elasticsearch
        metadata = self.get_metadata(product_identifier)

        # Initialize values dict
        values = {}
        values['id'] = product_identifier

        # Loop over all the property names and add them to the values if they are present
        for k, v in self.dict_info.items():
            tmp = self.extract_value(metadata, v)
            if tmp is not None:
                values[k] = tmp

        # Generate other properties based on extracted values
        values['creationtime'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        values['catingestiondate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())

        specific_infos = self.splitted_treatment(product_file_info, values)
        self.fix_id(values)

        # Generation of the output
        results = {}

        if self.jinja_template is None:
            self.with_template(self.template)

        for specific in specific_infos:
            result = self.jinja_template.render(
                dict=specific.values,
                dirpath=specific.product_file_info.get_product_path(),
                products=specific.product_file_info.datas,
                previews=specific.product_file_info.previews,
                metadatas=specific.product_file_info.metadatas,
                related=specific.product_file_info.related
            )
            results[specific.uid] = result

        return results

    def get_metadata(self, product_identifier):
        """ Get the product metadata from Elasticsearch
        :param product_identifier: product identifier
        :type product_identifier: str
        """
        return self.es_rest_support.get(self.es_index, product_identifier)

    def extract_value(self, metadata, path):
        """
        Extract a property, given by the path, from the metadata
        :param metadata: the metadata
        :type metadata: dict
        :param path: path of the of the property in the metadata dict
        :type path: str
        """
        path = path.split(".")
        tmp = metadata
        for key in path:
            if key in tmp:
                tmp = tmp[key]
            else:
                return None
        return tmp

    def fix_id(self, values):
        """ Fix the ID of the product by prepending the parent identifier
        :param values: dictionary containing the values, must include the product identifier and parent identifier
        :type values: dict
        """
        values['id'] = values['parentidentifier'] + ":" + values['title']

    def fix_geometry(self, values, simplify=True):
        """ Simplify the geometry and generate the bbox
        :param values: dictionary containing the values, must include the geometry
        :type values: dict
        """
        if "geometry" in values:
            polygon = EsaPolygon().from_geojson(values['geometry'], simplify=simplify)
        else:
            polygon = EsaPolygon().from_wkt(values['footprint'], simplify=simplify)
        values['geometry'] = polygon.as_geojson_string()
        values['westbbox'], values['southbbox'], values['eastbbox'], values['northbbox'] = polygon.get_bbox()

    def specific_treatment(self, product_file_info: ObjectInfo, values: dict):
        uid = values['title']
        if "_V" in uid:
            values['productVersion'] = uid[uid.rfind("_V")+1:]

        self.fix_products(product_file_info)

        return uid

    def fix_products(self, product_file_info):
        """" Helper method that moves files in the product file info to the correct section
        :param product_file_info: product file info
        :type product_file_info: ObjectInfo
        """
        raise NotImplementedError("Implement this method in a subclass.")

    def add_wms_quicklook(self, wms_layer: str, product_file_info: ObjectInfo, values: dict, timefield:str='endposition'):
        west, south = TRANSFORM_4326_TO_3857.transform(values['westbbox'], values['southbbox'])
        east, north = TRANSFORM_4326_TO_3857.transform(values['eastbbox'], values['northbbox'])

        # in WMS 1.3.0, the coordinate order is lat-lon
        wms_link = f"https://services.terrascope.be/wms/v2" \
                   "?SERVICE=WMS" \
                   "&REQUEST=getMap" \
                   "&VERSION=1.3.0" \
                   "&CRS=EPSG:3857" \
                   "&SRS=EPSG:3857" \
                   f"&LAYERS={wms_layer}" \
                   f"&TIME={values[timefield][:10]}" \
                   f"&BBOX={west},{south},{east},{north}" \
                   "&WIDTH=80" \
                   "&HEIGHT=80" \
                   "&FORMAT=image/png" \
                   "&TRANSPARENT=true"

        wms_quicklook = FileInfo(wms_link, file_type="image/png", file_title="WMS")
        product_file_info.previews.append(wms_quicklook)

    @staticmethod
    def format_uid_date(uid_date):
        return '{}-{}-{}T{}:{}:{}Z'.format(uid_date[0:4], uid_date[4:6], uid_date[6:8],
                                           uid_date[9:11], uid_date[11:13], uid_date[13:15])


TRANSFORM_4326_TO_3857 = pyproj.Transformer.from_crs("EPSG:4326", "EPSG:3857", always_xy=True)
